---
title : 版本管理
---

# Git

Git是一个开源的分布式版本控制系统，可以有效. 高速地处理从很小到非常大的项目版本管理。

## 组织结构管理

建立persagy组织，负责所有后台服务git代码管理。
组织下建立团队，包括公共项目团队、数字化交付、数据中台、集成框架、中控。
公共项目团队包括web前端、Android、IOS、WebRTC、U3D、Docker、artificial intelligence、framework等，
每一个公共项目代表一个技术领域。
所有业务团队公共，且与业务无关通用的代码采用公共项目团队内的git仓库管理。每个团队独立负责团队内git仓库建立。
每个git仓库对应一个idea的project，每个idea的project可以创建Module（模块）。
一般一个git仓库对应一个服务。
每个项目组构建组内的web项目，前台web项目也分层多个git仓库。
公司引入开源项目时，需要镜像开源项目的git仓库。引入的开源项目git镜像由公共项目团队管理。

目录结构如下:

```
└── persagy
    ├── Android
    ├── Docker
    ├── IOS
    ├── U3D
    ├── Web
    ├── WebRTC
    ├── artificial
    ├── framework
    ├── intelligence
    ├── 数字化交付
    │   ├── Android
    │   ├── IOS
    │   ├── Java
    │   └── Web
    ├── 数据中台
    │   ├── Android
    │   ├── IOS
    │   ├── Java
    │   └── Web
    ├── 集成框架
    │   ├── Android
    │   ├── IOS
    │   ├── Java
    │   └── Web
    └── 中控
        ├── Android
        ├── IOS
        ├── Java
        └── Web
```

## GitFlow 使用规则

在使用Git的过程中如果没有清晰流程和规划，否则,每个人都提交一堆杂乱无章的commit,项目很快就会变得难以协调和维护。
Git版本管理同样需要一个清晰的流程和规范。我们参照Gitflow 来规范我们的Git使用。

### Git Flow 流程图

![GitFlow流程图](/assets/liuchengtu.webp)

### Git Flow 的常用分支
- **Production 分支**
也就是我们经常使用的Master分支，这个分支最近发布到生产环境的代码，最近发布的Release， 这个分支只能从其他分支合并，不能在这个分支直接修改

- **Develop 分支**
这个分支是我们是我们的主开发分支，包含所有要发布到下一个Release的代码，这个主要合并与其他分支，比如Feature分支

- **Feature 分支**
这个分支主要是用来开发一个新的功能，一旦开发完成，我们合并回Develop分支进入下一个Release

- **Release分支**
当你需要一个发布一个新Release的时候，我们基于Develop分支创建一个Release分支，完成Release后，我们合并到Master和Develop分支

- **Hotfix分支**
当我们在Production发现新的Bug时候，我们需要创建一个Hotfix, 完成Hotfix后，我们合并回Master和Develop分支，所以Hotfix的改动会进入下一个Release


### Git Flow 如何使用

**Master/Devlop 分支**

所有在Master分支上的Commit应该打上Tag，一般情况下Master不存在Commit，Devlop分支基于Master分支创建

![Master/Devlop](/assets/Master.webp)

**Feature 分支**

Feature分支做完后，必须合并回Develop分支, 合并完分支后一般会删点这个Feature分支，毕竟保留下来意义也不大。

![Feature](/assets/Feature.webp)

**Release 分支**

Release分支基于Develop分支创建，打完Release分支之后，我们可以在这个Release分支上测试，修改Bug等。同时，其它开发人员可以基于Develop分支新建Feature (记住：一旦打了Release分支之后不要从Develop分支上合并新的改动到Release分支)发布Release分支时，合并Release到Master和Develop， 同时在Master分支上打个Tag记住Release版本号，然后可以删除Release分支了。

![Release](/assets/Release.webp)

**Hotfix 分支**

hotfix分支基于Master分支创建，开发完后需要合并回Master和Develop分支，同时在Master上打一个tag。

![Hotfix ](/assets/Hotfix.webp)

## Git Flow 命令示例

**创建 Devlop**

```
git branch develop  
git push -u origin develop
```

**开始 Feature**

```
# 通过develop新建feaeure分支
git checkout -b feature develop
# 或者, 推送至远程服务器:
git push -u origin feature    

# 修改md文件   
git status
git add .
git commit 
```

**完成 Feature**

```
git pull origin develop
git checkout develop 

#--no-ff：不使用fast-forward方式合并，保留分支的commit历史
#--squash：使用squash方式合并，把多次分支commit历史压缩为一次

git merge --no-ff feature
git push origin develop

git branch -d some-feature

# 如果需要删除远程feature分支:
git push origin --delete feature   
```

**开始 Release**

```
git checkout -b release-0.1.0 develop
```

**完成 Release**

```
git checkout master
git merge --no-ff release-0.1.0
git push

git checkout develop
git merge --no-ff release-0.1.0
git push


git branch -d release-0.1.0
git push origin --delete release-0.1.0   

# 合并master/devlop分支之后，打上tag 
git tag -a v0.1.0 master
git push --tags
```

**完成 Hotfix**

```
git checkout master
git merge --no-ff hotfix-0.1.1
git push


git checkout develop
git merge --no-ff hotfix-0.1.1
git push

git branch -d hotfix-0.1.1
git push origin --delete  hotfix-0.1.1 


git tag -a v0.1.1 master
git push --tags
```

## 版本号管理

- 第一位：产品大版本（与解决方案对应）（4.0开始）
- 第二位：研发产品线版本（产品线负责人确定）
- 第三位：研发发布版本（测试发布确认）(前端npm打包只有3位，提测及发布)
- 第四位：提测版本（研发确定）

Git打tag、分支都使用此版本号。对外发布版程序保留前3位，第四位只在研发、测试之间同步。

实例:
- 开发环境:4.1.1.2
- 生产环境:4.1.1
